import sys
import pytest
from .base_class import TestBase


from .test_jet_model import TestJets,hadronic_func
from .test_emitters import TestEmitters
from .test_ebl import TestEBL
from .test_depending_parameters import TestDependingParameters
from .test_composite_model import TestCompositeModel

@pytest.fixture
def plot():
   input = False
   return input


class TestUser(TestBase):
   
   def test_emitters(self,plot=plot):
      t=TestEmitters()
      t.integration_suite()

   def test_jet(self,plot=plot):
      t=TestJets()
      t.integration_suite()
    

   def test_dep_pars(self,plot=plot):
       t=TestDependingParameters()
       t.integration_suite()
   
   def test_composit_model(self,plot=plot):
       t=TestCompositeModel()
       t.integration_suite()

   def test_composit_ebl(self,plot=plot):
       t=TestEBL()
       t.integration_suite()

   def test_jet_hadronic(self,plot=plot):
      hadronic_func(plot)
      #t=TestJetHadronic()
      #t.test_hadronic_jet(plot=plot)


import  pytest
from .base_class import TestBase


class TestModelFit(TestBase):

    def test(self,plot=False):
       _= self.integration_suite(plot=plot,sed_number=1)

    def integration_suite(self,plot=False,sed_number=1,phenom_dict=None,use_dep_pars=False,use_ebl=False,skip_minuit=False):
        _=prepare_asset(plot=plot,sed_number=sed_number,phenom_dict=phenom_dict,use_dep_pars=use_dep_pars,use_ebl=use_ebl,skip_minuit=skip_minuit)
        

def prepare_asset(plot=False,sed_number=1,phenom_dict=None,use_dep_pars=False,use_ebl=False,skip_minuit=False):
    if sed_number is not None:
            template,jet,sed_data=prepare_model(plot=plot,sed_number=sed_number)
            print("===> template",template)
    elif phenom_dict is not None:
        template = phenom_dict['template']
        jet = phenom_dict['jet']
        sed_data = phenom_dict['sed_data']
    else:
        raise RuntimeError('please provide either sed_number or  phenom_dict')

    fit_model=prepare_fit_model(jet=jet,use_dep_pars=use_dep_pars,use_ebl=use_ebl,template=template)
    fit_model.show_model()
    fit_model, model_minimizer,sed_data=model_fit(fit_model=fit_model,
                                                        sed_data=sed_data, 
                                                        minimizer='lsb')


    if skip_minuit is False:
        fit_model, model_minimizer,sed_data=model_fit(fit_model=fit_model,
                                                            sed_data=sed_data, 
                                                            minimizer='minuit')

    return dict(fit_model=fit_model, model_minimizer=model_minimizer,sed_data=sed_data)



def prepare_model( plot=True,sed_number=0):
    from .test_phenom_constr import prepare_asset

    phenom_dict=prepare_asset(sed_number=sed_number,plot=plot)
    prefit_jet=phenom_dict['jet']
    sed_data=phenom_dict['sed_data']
    template=phenom_dict['template']
    prefit_jet.set_gamma_grid_size(200)
    print("===> template prepare model",template)
    return template,prefit_jet,sed_data

def prepare_fit_model(jet,use_dep_pars=False,use_ebl=False,template=None,minimizer='lsb'):
    from jetset.model_manager import FitModel


    fit_model = FitModel(jet=jet, name='SSC-best-fit-minuit', template=template)

    if use_ebl is True:
        from jetset.template_2Dmodel import EBLAbsorptionTemplate
        ebl_franceschini = EBLAbsorptionTemplate.from_name('Franceschini_2008')
        ebl_franceschini.show_model()
        fit_model.add_component(ebl_franceschini)
        fit_model.link_par(par_name='z_cosm', from_model='Franceschini_2008', to_model=jet.name)

    fit_model.freeze('jet_leptonic','z_cosm')
    fit_model.jet_leptonic.parameters.beam_obj.fit_range = [5, 50]
    fit_model.jet_leptonic.parameters.R_H.val = 5E17
    fit_model.jet_leptonic.parameters.R_H.frozen = False
    fit_model.jet_leptonic.parameters.R_H.fit_range = [1E15, 1E19]
    fit_model.jet_leptonic.parameters.R.fit_range = [10 ** 15.5, 10 ** 17.5]
    
    if use_dep_pars is True:
        if fit_model.parameters.get_par_by_name('jet_leptonic','B0') is None:
            fit_model.jet_leptonic.add_user_par(name='B0', units='G', val=1E3, val_min=0, val_max=None)
        
        if fit_model.parameters.get_par_by_name('jet_leptonic','R0') is None:
            fit_model.jet_leptonic.add_user_par(name='R0', units='cm', val=5E13, val_min=0, val_max=None)
        
        fit_model.jet_leptonic.parameters.R0.frozen = True
        fit_model.jet_leptonic.parameters.B0.frozen = True

        if fit_model.jet_leptonic.parameters.B._is_dependent  is False:
            fit_model.jet_leptonic.make_dependent_par(par='B', depends_on=['B0', 'R0', 'R_H'], par_expr='B0*(R0/R_H)')


    if use_ebl is True:
        if template is not None:
            fit_model.composite_expr = '(%s+host_galaxy)*Franceschini_2008'%jet.name
        else:
            fit_model.composite_expr = '(%s)*Franceschini_2008' % jet.name
    else:
        if template is not None:
            fit_model.composite_expr = '(%s+host_galaxy)'%jet.name
        

    

    if template is not None:
        fit_model.host_galaxy.parameters.nuFnu_p_host.frozen = False
        fit_model.host_galaxy.parameters.nu_scale.frozen = True

    
    return fit_model
    

def model_fit(fit_model,sed_data, plot=False,minimizer='lsb'):
    
    from jetset.minimizer import ModelMinimizer
    
    model_minimizer = ModelMinimizer(minimizer)
    
    if minimizer == 'minuit':
        fit_model.jet_leptonic.parameters.gmin.fit_range = [2, 300]
        fit_model.jet_leptonic.parameters.gmax.fit_range = [1E5, 1E7]

    best_fit = model_minimizer.fit(fit_model, sed_data, 10 ** 11., 10 ** 29.0,
                                                fitname='SSC-best-fit-%s'%minimizer, repeat=1)
    best_fit.show_report()
    best_fit.save_report('best-fit-%s-report.pkl'%minimizer)
    best_fit.bestfit_table.write('best-fit-%s-report.ecsv'%minimizer,overwrite=True)
    model_minimizer.save_model('model_minimizer_%s.pkl'%minimizer)

    model_minimizer = ModelMinimizer.load_model('model_minimizer_%s.pkl'%minimizer)
    fit_model.save_model('fit_model_%s.pkl'%minimizer)

    return fit_model, model_minimizer,sed_data

   


__author__ = "Andrea Tramacere"

import os
import numpy as np
#import copy
#import ctypes

on_rtd = os.environ.get('READTHEDOCS', None) == 'True'

if on_rtd == True:
    try:
        from .jetkernel import jetkernel as BlazarSED
    except ImportError:
        from .mock import jetkernel as BlazarSED
else:
    from .jetkernel import jetkernel as BlazarSED

from .jetkernel_models_dic import allowed_disk_type

from .jet_paramters import *


__all__=[ 'get_spectral_c_array']



def get_spectral_c_array(x_ptr, y_ptr, size, blob_object):
    x = np.zeros(size)
    y = np.zeros(size)

    for i in range(size):
        x[i] = BlazarSED.get_spectral_array(x_ptr, blob_object, i)
        y[i] = BlazarSED.get_spectral_array(y_ptr, blob_object, i)


    #if deep_copy is True:
    #    x = (ctypes.c_double * size).from_address(int(x_ptr))
    #    y = (ctypes.c_double * size).from_address(int(y_ptr))

    #x=copy.deepcopy(np.asarray(x))
    #y=copy.deepcopy(np.asarray(y)) 

    return x,y


import  pytest
from .base_class import TestBase
from .test_data import load_data_tool
class TestPhenomenologyConstr(TestBase):

    def integration_suite(self,sed_number=1,plot=False):
        prepare_asset(sed_number=sed_number,plot=plot)
        
        sed_data=load_data_tool(sed_number=sed_number,plot=False)
        my_shape=spectral_indices_tool(sed_data=sed_data,plot=plot,sed_number=sed_number)
        my_shape=sed_shaper_tool(my_shape=my_shape,sed_data=sed_data,sed_number=sed_number)
        prefit_jet, my_shape,template=model_constr_tool(my_shape=my_shape,sed_data=sed_data,plot=plot,sed_number=sed_number)
        
        return dict(jet=prefit_jet, shape=my_shape,template=template,sed_data=sed_data)

    def test_spectral_indices(self,sed_data=None,plot=True,sed_number=1):
        _=spectral_indices_tool(sed_data=sed_data,plot=plot,sed_number=sed_number)

    def test_sed_shaper(self,my_shape=None, plot=True, sed_number=1,sed_data=None,check_host_gal_template=False):
        _=sed_shaper_tool(my_shape=my_shape, plot=plot, sed_number=sed_number,sed_data=sed_data,check_host_gal_template=check_host_gal_template)

    def test_model_constr(self,my_shape=None, plot=True,sed_number=1,sed_data=None,check_host_gal_template=False):
        _=model_constr_tool(my_shape=my_shape, plot=plot,sed_number=sed_number,sed_data=sed_data,check_host_gal_template=check_host_gal_template)


def prepare_asset(sed_number=1,plot=False):
    sed_data=load_data_tool(sed_number=sed_number,plot=False)
    my_shape=spectral_indices_tool(sed_data=sed_data,plot=plot,sed_number=sed_number)
    my_shape=sed_shaper_tool(my_shape=my_shape,sed_data=sed_data,sed_number=sed_number)
    prefit_jet, my_shape,template=model_constr_tool(my_shape=my_shape,sed_data=sed_data,plot=plot,sed_number=sed_number)
    
    return dict(jet=prefit_jet, shape=my_shape,template=template,sed_data=sed_data)


def spectral_indices_tool(sed_data=None,plot=True,sed_number=1):
    from jetset.sed_shaper import SEDShape
    from .test_data import TestData
    if sed_data is None:
        sed_data=load_data_tool(sed_number=sed_number,plot=False)

    my_shape = SEDShape(sed_data)
    my_shape.eval_indices(silent=True)
    if plot is True:
        p = my_shape.plot_indices()
        p.setlim(y_min=1E-15, y_max=1E-6)

    return my_shape



def sed_shaper_tool(my_shape=None, plot=True, sed_number=1,sed_data=None,check_host_gal_template=False):
    if my_shape is None:
        my_shape=spectral_indices_tool(plot=plot,sed_number=sed_number,sed_data=sed_data)
    
    if sed_number == 2:
        check_host_gal_template=True
    
    print('==> check_host_gal_template',check_host_gal_template,sed_number)
    mm, best_fit = my_shape.sync_fit(check_host_gal_template=check_host_gal_template,
                                    Ep_start=None,
                                    minimizer='lsb',
                                    silent=True,
                                    fit_range=[10, 21])

    best_fit.show_report()
    best_fit.save_report('synch_shape_fit_rep.pkl')
    best_fit.bestfit_table.write('IC_shape_fit_rep.ecsv',overwrite=True)
    mm, best_fit= my_shape.IC_fit(fit_range=[23, 29], minimizer='minuit',silent=True)

    if plot is True:
        p = my_shape.plot_shape_fit()
        p.setlim(y_min=1E-15)

    best_fit.show_report()
    best_fit.save_report('IC_shape_fit_rep.pkl')
    best_fit.bestfit_table.write('IC_shape_fit_rep.ecsv',overwrite=True)
    my_shape.save_values('sed_shape_values.ecsv')
    return my_shape


def model_constr_tool(my_shape=None, plot=True,sed_number=1,sed_data=None,check_host_gal_template=False):
    from jetset.obs_constrain import ObsConstrain

    if sed_number == 2:
        check_host_gal_template=True
    print('==> check_host_gal_template', check_host_gal_template, sed_number)
    if my_shape is None:
        my_shape=sed_shaper_tool(plot=plot,sed_number=sed_number,sed_data=sed_data,check_host_gal_template=check_host_gal_template)

    sed_obspar = ObsConstrain(beaming=25,
                            B_range=[0.001, 0.1],
                            distr_e='lppl',
                            t_var_sec=3 * 86400,
                            nu_cut_IR=1E11,
                            SEDShape=my_shape)

    prefit_jet = sed_obspar.constrain_SSC_model(electron_distribution_log_values=False,silent=True)
    prefit_jet.save_model('prefit_jet.pkl')
    template=None
    if sed_number == 2:
        template = my_shape.host_gal
    return prefit_jet, my_shape, template


import pytest
from astropy import constants as const
from astropy import units as u
import matplotlib.pyplot as plt
import numpy as np
from .base_class import TestBase

class TestGalactic(TestBase):

    def integration_suite(self,plot=False):
        self.test_galactic_leptonic_unbeamed(plot=plot)
        self.test_galactic_hadronic_unbeamed(plot=plot)
        self.test_galactic_hadronic_beamed(plot=plot)

    def test_galactic_leptonic_unbeamed(self,plot=False):
        from jetset.jet_model import GalacticUnbeamed

        pwn=GalacticUnbeamed(emitters_distribution='bkn',verbose=False,emitters_type='electrons',distance=2*u.pc,name='pwn')
        pwn.add_EC_component('EC_CMB')
        pwn.parameters.R.val=(2*u.pc).to('cm').value
        pwn.parameters.B.val=1E-4
        pwn.parameters.gamma_break.val=1E8
        pwn.parameters.gmax.val=8E9
        pwn.parameters.gmin.val=5E5
        pwn.parameters.p.val=3.2
        pwn.parameters.p_1.val=3.8

        pwn.parameters.N.val=1E-8
        pwn.show_model()

        pwn.eval()
        if plot is True:
            p=pwn.plot_model(frame='src')
        
        pwn.energetic_report()
        assert('NH_cold_to_rel_e'  in pwn.energetic_dict.keys())
        _par_array=pwn._build_energetic_dict()
        np.testing.assert_allclose(pwn.energetic_dict['U_Synch'],pwn.energetic_dict['U_Synch_DRF'],rtol=1E-3)
        pwn.save_model('pwn.pkl')
        GalacticUnbeamed.load_model('pwn.pkl')

    def test_galactic_hadronic_unbeamed(self,plot=False):
        from jetset.jet_model import GalacticUnbeamed

        gal_hadronic=GalacticUnbeamed(emitters_distribution='plc',verbose=False,emitters_type='protons',distance=2*u.pc,name='gal_hadronic_unbeamed')
        gal_hadronic.parameters.R.val=1E18
        gal_hadronic.parameters.N.val=1000
        gal_hadronic.parameters.B.val=1E-3
        gal_hadronic.show_model()

        gal_hadronic.eval()
        if plot is  True:
            p=gal_hadronic.plot_model(frame='src')
            p.setlim(y_min=1E33)
        
        gal_hadronic.energetic_report()
        gal_hadronic.save_model('gal_hadronic.pkl')
        GalacticUnbeamed.load_model('gal_hadronic.pkl')

    def test_galactic_hadronic_beamed(self,plot=False):
        from jetset.jet_model import GalacticBeamed

        gal_hadronic=GalacticBeamed(emitters_distribution='plc',verbose=False,emitters_type='protons',distance=2*u.pc,name='gal_hadronic_beamed')
        gal_hadronic.parameters.R.val=1E18
        gal_hadronic.parameters.N.val=1000
        gal_hadronic.parameters.B.val=1E-3

        gal_hadronic.show_model()

        gal_hadronic.eval()
        
        if plot is True:
            p=gal_hadronic.plot_model(frame='src')
            p.setlim(y_min=1E36)

        gal_hadronic.energetic_report()
        gal_hadronic.save_model('gal_hadronic_beamed.pkl')
        GalacticBeamed.load_model('gal_hadronic_beamed.pkl')



__author__ = "Andrea Tramacere"

from astropy.units import  Unit as u
from astropy.units.quantity import Quantity
from astropy import cosmology 
from astropy.table import Table
from astropy.cosmology import Cosmology

import warnings

__all__=['Cosmo']


class  Cosmo (object):


    def __init__(self,astropy_cosmo=None,DL_cm=None,verbose=False):
        
        _c = None
        self._c_name=None
        if DL_cm is not None and astropy_cosmo is not None:
            raise  RuntimeError('Either you provide an astropy cosmology objet, or luminosity distance in cm, or nothing')

        elif  astropy_cosmo is None and DL_cm is None:
            _c = self._get_custom_cosmology()
            if verbose is True:
                print("cosmology set to custom",str(_c))
            

        elif astropy_cosmo is not None and DL_cm is None:
            _c = astropy_cosmo
            
        elif astropy_cosmo is None and DL_cm is not None:

            _c=None
            if verbose is True:
                print("using cosmo without z and only DL, should be used only for galactic objects!!")
                print("z will be fixed to zero")
        else:
            raise RuntimeError('Either you provide an astropy cosmology objet, or luminosity distance in cm, or nothing')


        self._c = _c
        if DL_cm is not None:
            if isinstance(DL_cm,Quantity):
                if u(DL_cm)=='cm':
                    pass
                else:
                    DL_cm=DL_cm.to('cm')
            else:
                DL_cm=DL_cm*u('cm')
        self._DL_cm = DL_cm

    def __repr__(self):
        if self._c is not None:
            s= '%s' % self._c
        elif self._DL_cm is not None:
            x=self._DL_cm

            s= 'cosmology is not defined, the luminosity distance has been set to %s'%x
        else:
            s='you have to define either an astropy cosmology model or set a luminosity distance in cm'

        return s

    def get_DL_cm(self,z=None):
        if self._c is not None:
            #THIS IS FIXING THE ERROR WITH PICKLED COSMO
            #TODO: open issue on astropy!
            try:
                _d= self._c.luminosity_distance( z ).to('cm').value
            except:
                _d = self._c.luminosity_distance(z)
                _d = _d.value*u(str(_d.unit))
                _d = _d.to('cm').value
        else:
            _d = self._DL_cm.value

        return _d
    
    def _serialize_model(self):
        _model = {}
        if self._c is not None:
            _model['_astropy_cosmo']=Table(self._c.to_format('astropy.table'))
        else:
            _model['_astropy_cosmo']=None
            _model['_DL_cm'] = self._DL_cm.value
        return   _model 
          
    def __getstate__(self):
        return  self._serialize_model()

    def __setstate__(self,state):
        astropy_cosmo,DL_cm=self._decode_model(state)
        self.__init__(astropy_cosmo=astropy_cosmo,DL_cm=DL_cm)
    
    @classmethod
    def from_model(cls,model):
        astropy_cosmo,DL_cm  = cls._decode_model(model)
        return cls(astropy_cosmo,DL_cm)

    @staticmethod
    def _get_custom_cosmology():
        try:
            return cosmology.Planck13
        except:
            return cosmology.FlatLambdaCDM(H0=67.8, Om0=0.307)


    @staticmethod
    def _decode_model(model):
        DL_cm=None
        astropy_cosmo=None
        try:
            if '_astropy_cosmo' in model.keys():
                if model['_astropy_cosmo'] is not None:
                    try:
                        astropy_cosmo=Cosmology.from_format(Table(model['_astropy_cosmo']),format='astropy.table')
                    except Exception as e:
                        warnings.warn('unable to get astropy.cosmology from loaded instance, setting to Planck13')
                        astropy_cosmo = cosmology.Planck13
            if '_DL_cm' in model.keys():
                DL_cm=model['_DL_cm']
        except Exception as e:
            warnings.warn('failed to decode saved astropy model, reason: %s'%str(e))       
        return astropy_cosmo,DL_cm  




import sys
import io
import pip
from contextlib import redirect_stdout
from packaging import version
import subprocess

class capture(redirect_stdout):

    def __init__(self):
        self.f = io.StringIO()
        self._new_target = self.f
        self._old_targets = []  # verbatim from parent class

    def __enter__(self):
        self._old_targets.append(getattr(sys, self._stream))  # verbatim from parent class
        setattr(sys, self._stream, self._new_target)  # verbatim from parent class
        return self  # instead of self._new_target in the parent class

    def __repr__(self):
        return self.f.getvalue()


def check_version():
    try:
        p = subprocess.run(['python', '-m','pip', 'index', '--retries', '1', 'versions', 'jetset'], capture_output=True, text=True,timeout=1)
        message = p.stdout
    except:
        p.kill()
        outs, errs = p.communicate()

    lines=str(message).split('\n')
    installed= None
    latest = None
    for line in lines:
        if 'INSTALLED:' in line:
            installed=line.split(':')[1].strip()
        if 'LATEST:' in line:
            latest=line.split(':')[1].strip()
    if installed is not None and latest is not None:
        if  version.parse(installed) < version.parse(latest):
            o_message=('there is a new jetset version %s\n '%latest)
            o_message+=('there installed one is %s',installed)
        else:
            o_message='jetset version is up to date v:%s'%installed
    else:
        o_message='not able to get latest version info from pip'
    
    print(o_message)

def run_version_checking():
    try:
        check_version()
    except Exception as e:
        print("can not establish internet connection, I can't verify if jetset version is up to date")



__author__ = "Andrea Tramacere"

import os
try:
    from gammapy.modeling.models import (
    SpectralModel,
)
    from gammapy.modeling.parameter import Parameter,Parameters
    from gammapy.estimators import FluxPoints
    from gammapy.datasets import FluxPointsDataset
    from gammapy.modeling import Fit

    gammapy_installed = True

except:
    on_rtd = os.environ.get('READTHEDOCS', None) == 'True'
    
    if on_rtd is True:
        SpectralModel=object
        pass
    else:
        raise  ImportError('to use gammapy plugin you need to install gammapy: https://docs.gammapy.org/0.19/getting-started/install.html')

import astropy.units as u
import  numpy as np

__all__=['GammapyJetsetModel','GammapyJetsetModelFactory']

class GammapyJetsetModel(SpectralModel):
    

    def __init__(self,jetset_model,clone=True):
       
        if clone is True:
            _jetset_model = jetset_model.clone()
        else:
            _jetset_model= jetset_model
        
        self._jetset_model=_jetset_model
    
        self._jetset_model.add_user_par(name='fake_norm',units='',val=1,val_min=0,val_max=None)
        self._jetset_model.parameters.fake_norm.frozen=True
        parameters = []
        
        for ID,p in enumerate(self._jetset_model.parameters.par_array):
            #print(p.name)
            if p.name=='fake_norm':
                is_norm=True
            else:
                is_norm=False

            parameter = Parameter(p.name, p.val, is_norm=is_norm,frozen=p.frozen)
            if _jetset_model.parameters.par_array[ID].units is not None:
                try:
                    parameter.unit = p.units
                except:
                    parameter.unit = ''
            else:
                  parameter.unit = ''


            if p.val_min is not None:
                parameter.min=p.val_min
            
            if p.fit_range_min is not None:
                 parameter.min=p.fit_range_min

            if p.val_max is not None:
                parameter.max=p.val_max
            
            if p.fit_range_max is not None:
                 parameter.max=p.fit_range_max

            parameters.append(parameter)
        self.default_parameters = Parameters(parameters)
        self.tag=_jetset_model.name
        super(GammapyJetsetModel, self).__init__()

    def evaluate(self,energy=None,**kwargs):

        if energy is None:
            el1=np.log10( self._jetset_model.nu_min)
            el2=np.log10( self._jetset_model.nu_max)
            energy=(np.logspace(el1,el2,self._jetset_model.nu_size)*u.Hz).to('eV',equivalencies=u.spectral())
        
        nu = energy.to("Hz", equivalencies=u.spectral())

        for p in self.parameters:
            if p.name not in kwargs.keys():
                self._jetset_model.set_par(p.name ,val=p.value)

        for k,v in kwargs.items():
            self._jetset_model.set_par(k,val=v.value)

        self._jetset_model.eval(nu=nu.value)
        _spec= self._jetset_model.spectral_components.Sum.SED.nuFnu.to('eV cm-2 s-1')/(energy.to('eV')**2)
        return _spec.to("1 / (cm2 eV s)")
    
    @property
    def jetset_model(self):
        return self._jetset_model
    

def GammapyJetsetModelFactory(jetset_model,clone=True):
    return GammapyJetsetModel(jetset_model,clone=clone)


import pytest
import os
from .base_class import TestBase

from .test_mcmc import TestEmcee
from .test_jet_model import TestJets,hadronic_func
from .test_model_fit import TestModelFit
from .test_emitters import TestEmitters
from .test_ebl import TestEBL
from .test_mcmc import TestEmcee
from .test_depending_parameters import TestDependingParameters
from .test_composite_model import TestCompositeModel
from .test_temp_ev import TestTempEv
from .test_galactic import TestGalactic

@pytest.fixture
def plot():
   input = False
   return input

class TestIntegration(TestBase):

   def test_jet(self,plot=plot):
      t=TestJets()
      t.integration_suite()
   
   @pytest.mark.skipif(os.getenv('WF_ENV')=='CONDA', reason="not running with conda") 
   def test_jet_hadronic(self,plot=plot):
       hadronic_func(plot)
   #   #t=TestJetHadronic()
   #   #t.test_hadronic_jet(plot=plot)
   
   @pytest.mark.skipif(os.getenv('WF_ENV')=='CONDA', reason="not running with conda") 
   def test_galactic(self,plot=plot):
      t=TestGalactic()
      t.integration_suite(plot=plot)

   def test_emitters(self,plot=plot):
      t=TestEmitters()
      t.integration_suite(plot=plot)

   def test_dep_pars(self,plot=plot):
      t=TestDependingParameters()
      t.integration_suite(plot=plot)
   
   def test_composit_model(self,plot=plot):
      t=TestCompositeModel()
      t.integration_suite(plot=plot)

   def test_composit_ebl(self,plot=plot):
      t=TestEBL()
      t.integration_suite(plot=plot)

      
   def test_model_fit(self,phenom_dict=None,plot=plot):
      from .test_phenom_constr import prepare_asset
      phenom_dict=prepare_asset(sed_number=1)
      t=TestModelFit()
      t.integration_suite(sed_number=None,phenom_dict=phenom_dict,use_ebl=False,use_dep_pars=False,skip_minuit=True,plot=plot)
   

   def test_emcee(self,fit_dict=None,plot=plot):
      t=TestEmcee()
      if fit_dict is None:
         sed_number=1
      else:
         sed_number=None
      t.integration_suite(fit_dict=fit_dict,sed_number=sed_number,plot=plot)

   def test_temp_ev(self,plot=plot):
      t=TestTempEv() 
      t.integration_suite(plot=plot)  


